--[[
    
***************************************************************
Created by: mrfatpockets (1dabread)
Date: Today
Description:
    Add the poisonable component to 
TODO: poisonfx - Copy from firefx
***************************************************************

]]--
local prefabs = 
{
    "poison"
}

function c_testpoison(dopoison, dmg, time) 
    local inst = TheInput:GetWorldEntityUnderMouse()
    if inst.components.poisonable then
        local p = inst.components.poisonable
        print("IS POISONABLE | " .. "STATE: " .. p.state .. " | SICKNESS: " .. (p.sickness or 0) .. " | RESIST: " .. p:GetResistance().. " | ARMOR: " .. p:GetSicknessArmor())
        print("HEALTH: " .. inst.components.health.currenthealth)
    if dopoison then 
        print("POISONING...")
        p:Poison(dmg, time)
    end
    end
end

function c_whoami()
    local inst = TheInput:GetWorldEntityUnderMouse()
    print(inst.prefab)
end

local states = 
{
    POISONED = "POISONED",
    NORMAL = "NORMAL",
}

local function hurtFromPoison(inst)
    local h = inst.components.health
    local p = inst.components.poisonable
    if h then
        if h:IsDead() then
            return
        end
        if p and p.state == states.POISONED then
            local psn_dmg = (p.poisondmg or 5)
            local dt = math.max(0,psn_dmg - (psn_dmg * p:GetSicknessArmor()))
            --print(inst.prefab .. ":: HEALTH = " .. inst.components.health.currenthealth .. " | DAMAGE = " .. dt)
            h:DoDelta(-dt,true,"poisoned")
            
            local pos = Vector3(inst.Transform:GetWorldPosition())
            pos.y = pos.y + 1 + math.random()*1.5
            local poison = SpawnPrefab("poison")
            poison.Transform:SetPosition(pos:Get())
        end
    end

    if p and p.sickness <= 0 then
        p:UnPoison()
    else
        p.sickness = p.sickness - 1
    end
end

local function Cured(inst)
    inst.components.poisonable.sickness = 0
end
-----------------------------------------------------------------------------------------------------

local Poisonable = Class(function(self, inst)
    self.inst = inst
    self.state = states.NORMAL
    self.resistance = nil
    self.resistBonus = 0
    self.sickness = nil
    self.sicknessArmor = 0
    self.armorBonus = 0
    self.poisondmg = nil
    self.fxlevel = 1
    self.fxdata = {}
    self.fxchildren = {}
    self.task = nil
end)

function Poisonable:SetResistanceBonus(resist)
    if resist then self.resistBonus = resist end
end

function Poisonable:GetResistance()
    return (self.resistance or 0) + self.resistBonus
end

function Poisonable:SetSicknessArmorBonus(armor)
    if armor then self.armorBonus = armor end
end

function Poisonable:GetSicknessArmor()
    return self.sicknessArmor + self.armorBonus
end

function Poisonable:GetSickness()
    return (self.sickness or 0)
end

function Poisonable:AddShatterFX(prefab, offset, followsymbol)
    table.insert(self.fxdata, {prefab=prefab, x = offset.x, y=offset.y, z=offset.z, follow=followsymbol})
end

function Poisonable:SetShatterFXLevel(level, percent)
    self.fxlevel = level

    for k,v in pairs(self.fxchildren) do
        if v.components.shatterfx then
            v.components.shatterfx:SetLevel(level)
        end
    end
end

function Poisonable:SpawnShatterFX()
    for k,v in pairs(self.fxdata) do
        local fx = SpawnPrefab(v.prefab)
        if fx then
            if v.follow then
                local follower = fx.entity:AddFollower()
                follower:FollowSymbol(self.inst.GUID, v.follow, v.x,v.y,v.z)
            else
                self.inst:AddChild(fx)
                fx.Transform:SetPosition(v.x, v.y, v.z)
            end
            table.insert(self.fxchildren, fx)
            if fx.components.shatterfx then
                fx.components.shatterfx:SetLevel(self.fxlevel)
            end
        end
    end
end

function Poisonable:IsPoisoned()
    return self.state == states.POISONED
end

function Poisonable:AddSickness(amt,dmg)
   self.sickness = amt + (self.sickness or 0)
    if self.task then
        self.task:Cancel()
        self.task = nil
    end
    if not self:IsPoisoned() then
        if self.sickness <= self:GetResistance() then
            self.task = self.inst:DoTaskInTime(10, Cured)
        else
            self:Poison(math.max(10,self.sickness),dmg)
        end
    end
end

function Poisonable:StartPoisonDMG()
    if self.poisondmgtask then
        self.poisondmgtask:Cancel()
        self.poisondmgtask = nil
    end
    self.poisondmgtask = self.inst:DoPeriodicTask(1, hurtFromPoison)
end

function Poisonable:Poison(amt,dmg)
    if self.inst.entity:IsVisible() and not (self.inst.components.health and self.inst.components.health:IsDead()) and not self:IsPoisoned() then
        self.state      = states.POISONED
        self.poisondmg  = (dmg or  5)
        self.sickness   = (amt or 10)

        self:StartPoisonDMG()

        self.inst:PushEvent("poisoned")

        self.inst.AnimState:SetMultColour(.5, 1, .5, 1)
    end
    
end

function Poisonable:UnPoison()
    if (not self.inst.components.health or not self.inst.components.health:IsDead()) and self:IsPoisoned() then
        if self.poisondmgtask then
            self.poisondmgtask:Cancel()
            self.poisondmgtask = nil
        end
        if self.task then
            self.task:Cancel()
            self.task = nil
        end
        self.state      = states.NORMAL
        self.sickness   = nil
        self.poisondmg  = nil

        self.inst:PushEvent("unpoison")

        self.inst.AnimState:SetMultColour(1, 1, 1, 1)
    end
end

function Poisonable:OnSave()
    local data = {} 
    data.poison_sickness = self.sickness
    data.poison_state    = self.state
    data.poison_dmg      = self.poisondmg
    return data
end

function Poisonable:OnLoad(data)
    local sickness  = (data.poison_sickness or self.sickness or 0)
    local poisondmg = (data.poison_dmg or self.poison_dmg or nil)
    local state     = (data.poison_state or states.NORMAL)

    -- 0 time schedule is used because OnLoad is done before AddSimPostInit I guess
    self.inst:DoTaskInTime(0,function()
            if sickness > 0 then
                --print(self.inst.prefab.." | SICKNESS: "..sickness.." | STATE: "..state.." | DMG: "..poisondmg)
                if state == states.POISONED then
                    --print("POISONED!!!!!")
                    self:Poison(sickness,poisondmg)
                else
                    --print("SICKNESS ADDED!!!!!")
                    self:AddSickness(sickness,poisondmg)
                end
            end
        end)

end

return Poisonable