

local assets=
{
    Asset("ANIM", "anim/trap_beespring.zip"),                   -- Trap
    Asset("IMAGE", "images/inventoryimages/trap_beespring.tex"),-- Inventory Image
    Asset("ATLAS", "images/inventoryimages/trap_beespring.xml"),
}

local function onfinished(inst)
    inst:RemoveComponent("inventoryitem")
    inst:RemoveComponent("mine")
    inst.persists = false
    inst.AnimState:PushAnimation("used", false)
    inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
    inst:DoTaskInTime(3, function() inst:Remove() end )
end

local function onExplode(inst, target)
    inst.AnimState:PlayAnimation("trap")
    if target then
        inst.SoundEmitter:PlaySound("dontstarve/common/trap_teeth_trigger")
        target.components.combat:GetAttacked(inst, (20)*dmgmultiplier)
        -- Poisoned: damage, ticks
        if target.components.poisonable then
            if not target.components.poisonable:IsPoisoned() then 
                target.components.poisonable:Poison(10,8)
            else
                target.components.poisonable:AddSickness(10,8)
            end
        end
        if METRICS_ENABLED then
            FightStat_TrapSprung(inst,target, (20)*dmgmultiplier)
        end
    end
    if inst.components.finiteuses then
        inst.components.finiteuses:Use(1)
    end
end

local function onReset(inst)
    inst.SoundEmitter:PlaySound("dontstarve/common/trap_teeth_reset")
    inst.AnimState:PlayAnimation("reset")
    inst.AnimState:PushAnimation("idle", false)
end

local function setSprung(inst)
    inst.AnimState:PlayAnimation("trap_idle")
end

local function setInactive(inst)
    inst.AnimState:PlayAnimation("inactive")
end

local function onDropped(inst)
    inst.components.mine:Deactivate()
end

local function ondeploy(inst, pt, deployer)
    inst.components.mine:Reset()
    inst.Physics:Teleport(pt:Get())
end

--legacy save support - mines used to start out activated
local function onload(inst, data)
    if not data or not data.mine then
        inst.components.mine:Reset()
    end
end

local function fn()
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    MakeInventoryPhysics(inst)
    
    local minimap = inst.entity:AddMiniMapEntity()
    minimap:SetIcon( "toothtrap.png" )
   
    inst.AnimState:SetBank("trap_teeth")
    inst.AnimState:SetBuild("trap_beespring")
    anim:PlayAnimation("idle")
    
    inst:AddTag("trap")
    
    inst:AddComponent("inspectable")
    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.nobounce = true
    inst.components.inventoryitem:SetOnDroppedFn( onDropped )
    inst.components.inventoryitem.atlasname = "images/inventoryimages/trap_beespring.xml"

    inst:AddComponent("mine")
    inst.components.mine:SetRadius(TUNING.TRAP_TEETH_RADIUS)
    inst.components.mine:SetAlignment("player")
    inst.components.mine:SetOnExplodeFn( onExplode )
    inst.components.mine:SetOnResetFn( onReset )
    inst.components.mine:SetOnSprungFn( setSprung )
    inst.components.mine:SetOnDeactivateFn( setInactive )
    
    inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetMaxUses(TUNING.TRAP_TEETH_USES * usemultiplier)
    inst.components.finiteuses:SetUses(TUNING.TRAP_TEETH_USES * usemultiplier)
    inst.components.finiteuses:SetOnFinished( onfinished )
    
    inst:AddComponent("deployable")
    inst.components.deployable.ondeploy = ondeploy
    inst.components.deployable.min_spacing = .75
    
    inst.components.mine:Deactivate()
    inst.OnLoad = onload
    return inst
end

return Prefab( "common/inventory/trap_beespring", fn, assets)
        --MakePlacer("common/trap_teeth_placer", "trap_teeth", "trap_teeth", "idle")